*-----------------------------------------------------------
* Title      : PGM Snake Demo
* Written by : IQ_132
* Date       : April 8, 2023
* Description: Test various features of the PGM hardware
*-----------------------------------------------------------

base_address			equ		$100000	; change to 0 if booting over bios
sprite_offset			equ		$2280	; which sprite to use? 4500/2 in happy 6 in 1

addr_main_ram			equ		$800000
addr_stack				equ		$820000
addr_bg_ram				equ		$900000
addr_char_ram			equ		$904000
addr_palette			equ		$a00000
addr_text_palette		equ		$a01000
addr_zoom_table			equ		$b01000
addr_fg_scrolly			equ		$b05000
addr_fg_scrollx			equ		$b06000
addr_scanline_counter	equ		$b07000
addr_video_flags		equ		$b0e000

; Writes to $b0e000
;
;	0000 0000 0000 000x - sprite dma enable - pulse 0->1 to trigger (does 1->0 work?)
;	0000 0000 0000 00x0 - nothing?
;	0000 0000 0000 0x00 - IRQ4 ACK - pulse to trigger (0->1 and 1->0 works?)
;	0000 0000 0000 x000 - IRQ6 ACK - pulse to trigger (0->1 and 1->0 works?)
;	0000 0000 000x 0000 - all games set this?
;	0000 0000 0xx0 0000 - all games except CAVE, but seems to serve no purpose when set?
;	0000 0000 x000 0000 - causes system to lose video synch?
;	0000 000x 0000 0000 - shows garbage on screen for all except background?
;	0000 00x0 0000 0000 - disable everything except background layer?
;	0000 0x00 0000 0000 - nothing?
;	0000 x000 0000 0000 - disable text layer
;	000x 0000 0000 0000 - disable background layer
;	00x0 0000 0000 0000 - disable high priority sprites
;	xx00 0000 0000 0000 - nothing?

addr_player_inputs		equ		$c08000

pos_player_x			equ		$800000
pos_player_y			equ		$800002

pos_fruit_y				equ		$810000
pos_fruit_x				equ		$810002
pos_fruit_y_prev		equ		$810004
pos_fruit_x_prev		equ		$810006
pos_fruit_y_rng			equ		$810008
pos_fruit_x_rng			equ		$81000a

inputs_previous			equ		$81000c
inputs_current			equ		$81000e

frame_counter			equ		$810010 ; long

player_score			equ		$810014

manual_vlk_trigger		equ		$810020

START:                  ; first instruction of program

	org		base_address+$0

	dc.l	addr_stack	; sp
	dc.l	fn_init	; initial vector

	; interrupt vectors
	org		base_address+$60
	dc.l	fn_irq_spurious
	dc.l	fn_irq1	; not used?
	dc.l	fn_irq2 ; not used?
	dc.l	fn_irq3 ; not used?
	dc.l	fn_irq4 ; cart-controlled
	dc.l	fn_irq5 ; not used?
	dc.l	fn_irq6	; vblank
	dc.l	fn_irq7 ; not used?

	org 	base_address+$200
	dc.b	'IGS PGM PLATFORM GAMES',0,0,0,0,0,0,0,0,0,0	; this string is tested by the bios, non-boot without it
	dc.b	'SNAKE',0,0,0,0,0,0,0,0,0,0,0,0
	dc.b	'V0.2',0,0,0,0,0,0

	org 	base_address+$23a
	dc.l	fn_initialize_fn											; init vector from the bios??

	dc.b	'04/08/23', 0, 0	; most games set date date here
	dc.b 	'01:32:00'			; most games set build time here

; initialization routine
; very similar to the USER routine in Neo-Geo
fn_initialize_fn
	; same w/py2k2, photoy2k, kov2, dmnfront, probably others
	; link    A6, #$0
	; move.l  D2, -(A7)

	; move.w ($8,A6), D2	; select what this function does...
	; if (D2 == 0) initialize (set up) NV RAM
	; if (D2 == 1) operator setting menu (soft dips)
	; if (D2 == 2) seems to do nothing? (bra.s	lb_skip_initialize)
	; if (D2 == 3) seems to do nothing?	(bra.s	lb_skip_initialize)
	; if (D2 >= 4) skip initialize ?
lb_skip_initialize
	; move.l  (A7)+, D2
	; unlk    A6
	rts

;	org	$400
fn_irq_spurious
;	move.w	#$38, $905b00
;	move.w	#$2, $905b02
	rte

fn_irq1
;	move.w	#$39, $905b04
;	move.w	#$2, $905b06
	rte

fn_irq2
;	move.w	#$3a, $905b08
;	move.w	#$2, $905b0a
	rte

fn_irq3
;	move.w	#$3b, $905b0c
;	move.w	#$2, $905b0e
	rte

fn_irq4
	move.w  #$007b, addr_video_flags
	nop
	move.w  #$107f, addr_video_flags
	andi.w	#$f0ff, SR
;	move.w	#$3c, $905b10
;	move.w	#$2, $905b12
	rte

fn_irq5
;	move.w	#$3d, $905b14
;	move.w	#$2, $905b16
	rte

fn_irq7
;	move.w	#$3f, $905b1c
;	move.w	#$2, $905b1e
	rte

fn_irq6
;	clr.w	$700006	; on-cart watchdog?
	move.w	inputs_current, inputs_previous
	move.w	addr_player_inputs, inputs_current
	addi.l	#$1, frame_counter

	move.w  #$0077, addr_video_flags	; 1000 disable bg layer, pulse (0 << 3) for IRQ6 ack

	move.w	inputs_current, D0
	move.w	inputs_previous, D1

	move.w  #$107f, addr_video_flags	; 1000 disable bg layer, pulse (1 << 3) for IRQ6 ack

	bsr		fn_game_logic

	; no wait required for this pulse to register and start dma'ing the sprites?
	move.w  #$107e, addr_video_flags	; 1000 for disable bg layer, pulse (0 << 0) for sprite buffer
	move.w  #$107f, addr_video_flags	; 1000 disable bg layer, pulse (0 << 0) for sprite buffer

	andi.w	#$f0ff, SR
	rte



fn_init
	move.w	#$2700, SR		; disable all interrupts
	lea.l	addr_stack, A7	; set stack pointer

	move.w  #$007e, addr_video_flags

	move.l	#$0, manual_vlk_trigger

	lea.l	addr_bg_ram, A0	; bg ram
	move.w	#$3ff, D0
	bsr	fn_clear_ram

	lea.l	addr_char_ram, A0	; text ram
	move.w	#$7ff, D0
	bsr	fn_clear_ram

	lea.l	addr_main_ram, A0	; sprite list
	move.w	#$27f, D0
	bsr	fn_clear_ram

	lea.l	addr_palette, A0	; palette
	move.w	#$4ff, D0
	bsr	fn_clear_ram

;	bsr		fn_blank_text
;	bsr		fn_blank_bg
;	bsr		fn_blank_sprite
;	bsr		fn_clear_palette

	bsr		fn_fill_zoom_table
	bsr		fn_border

	move.w	#$0, addr_fg_scrolly+$800	; set text layer scroll offsets (add $800 to demonstrate that these are mirrored)
	move.w	#$0, addr_fg_scrollx+$800

	move.w	#$f, D0
	move.l  #$7fff7fff, D1

	lea.l	addr_palette, A0
lb_fill_spr_palette
	move.l	D1, (A0)+  ; white sprite
	dbra	D0, lb_fill_spr_palette

	move.w	#$8000, addr_palette+$7fe	; black background!

	lea.l   addr_text_palette, A0

	; color 0 (0 << 1) - white text layer
	move.l	D1, (A0)+ 
	move.l	D1, (A0)+
	move.l	D1, (A0)+
	move.l	D1, (A0)+
	adda.w	#$10, A0

	; color 1 (1 << 1) - white w/blue border
	move.l  D1, (A0)+
	move.l  D1, (A0)+
	move.l  D1, (A0)+
	move.l  #$7fff241f, (A0)+
	adda.w	#$10, A0

	move.w	#$1e, pos_fruit_y

	bsr		fn_game_reset

	; initial fruit position
	bsr		fn_update_fruit_pos

	move.w	#$2000, SR		; enable all interrupts

; test on-board character printing
;	lea.l	lb_testtext, A0
;	lea.l	addr_char_ram, A1
;	move.w	#$2, D0
;	bsr		fn_print_quad_size

	; wait for vblank to run game logic
lb_wait_loop
	bra.s	lb_wait_loop



fn_game_logic
	moveq.l	#$0, D2

	; check button presses - this cound be cleaner
	btst	#$1, D0	; up
	bne.s	lb_button_up_np
		moveq.l	#$1, D2
		subi.w	#$1, pos_player_y
lb_button_up_np

	btst	#$2, D0	; down
	bne.s	lb_button_dn_np
		moveq.l	#$1, D2
		addi.w	#$1, pos_player_y
lb_button_dn_np

	btst	#$3, D0	; left
	bne.s	lb_button_lt_np
		moveq.l	#$1, D2
		subi.w	#$1, pos_player_x
lb_button_lt_np

	btst	#$4, D0	; right
	bne.s	lb_button_rt_np
		moveq.l	#$1, D2
		addi.w	#$1, pos_player_x
lb_button_rt_np

	andi.w	#$81ff, pos_player_y
	andi.w	#$83ff, pos_player_x

	tst.b	D2
	beq		lb_no_hit

lb_check_positions

	move.w	pos_player_y, D0
	move.w	pos_player_x, D1
	andi.w	#$1ff, D0
	andi.w	#$3ff, D1

;----------------------------------------------
; check for player hitting the wall

	cmpi.w	#$8, D0
	blt		lb_game_over		; hit wall

	cmpi.w	#$8, D1
	blt		lb_game_over		; hit wall

	cmpi.w	#$d0, D0
	bge		lb_game_over		; hit wall

	cmpi.w	#$1a8, D1
	bge		lb_game_over		; hit wall

;----------------------------------------------
; check for player hitting the fruit

	move.w	pos_fruit_y, D2
	move.w	pos_fruit_x, D3
	lsl.w	#$3, D2
	lsl.w	#$3, D3
	move.l	#$00100008, D4 ; sprite high = size x, low = size y
	move.l	#$00080008, D5 ; fruit high = size x, low = size y

	bsr		fn_check_collision
	beq.s	lb_no_hit

		; update player score
		move.w	player_score, D0
		move.w	#$1, D1
		abcd	D1, D0
		move.w	D0, player_score

		; print score to screen
		lea.l	addr_char_ram+$68, A0		; (0 * 256) + (((448/8/2)-2)*4)
		moveq.l	#$2, D1
		moveq.l	#$4, D2
		bsr 	fn_print_hex

		; update fruit position
		bsr		fn_update_fruit_pos

lb_no_hit
	rts

lb_game_over
	bsr.s 	fn_game_reset
	bra		fn_update_fruit_pos



fn_game_reset
	move.w	#$ffff, inputs_current
	move.w	#$ffff, inputs_previous
;	move.w	#$1e, pos_fruit_y
;	move.w	#$0, pos_fruit_x
;	move.w	#$0, pos_fruit_y_prev
;	move.w	#$0, pos_fruit_x_prev
;	move.w	#$0, pos_fruit_y_rng
;	move.w	#$0, pos_fruit_x_rng
	move.l	#$0, frame_counter
	move.w	#$0, player_score

	; print score to screen
	moveq.l	#$0, D0
	lea.l	addr_char_ram+$68, A0		; (0 * 256) + (((448/8/2)-2)*4)
	moveq.l	#$2, D1
	moveq.l	#$4, D2
	bsr 	fn_print_hex

	; configure player sprite
	; start player at (near) center)
	move.w	#$80d8, pos_player_x				; 0 - 448/2-8f
	; xzoom = 7800
	; xgrow = 8000 (enable x zoom)
	move.w	#$8068, pos_player_y				; 2 - 224/2-8
	; yzoom = 7800
	; ygrow = 8000 (enable y zoom)
	move.w	#$0000, addr_main_ram+$4
	; sprite (high) - 007f
	; priority      - 0080 ; 1 = behind everything, 0 = above
	; palette color	- 1f00
	; flip			- 6000
	move.w	#sprite_offset, addr_main_ram+$6		; 6 - sprite offset (low word)
	move.w	#$0208, addr_main_ram+$8		; 8 - size is (8 << 9) | 8; // x = 8, y = 8
	; wide = 7e00
	; high = 01ff
	move.w	#$0, addr_main_ram+$12			; set end of sprite list marker
	rts


; 448x224 border
fn_border
	movem.l	D0-D2/A0, -(A7)
	move.w	#$1f, D2
	lea.l	addr_char_ram, A0 ; each row - 64 * 4
	move.w	#$20, D0	; text character
	move.w	#$0, D1		; color
lb_border_column
		move.w	D0, ($0,A0)	; left
		move.w	D1, ($2,A0)
		move.w	D0, ($dc,A0)	; right
		move.w	D1, ($de,A0)
		adda.l	#$100, A0
	dbra	D2, lb_border_column

	move.w	#$37, D2
	lea.l	addr_char_ram, A0
	; use D1,D0 from above...
lb_border_row
		move.w	D0, ($0,A0)	; left
		move.w	D1, ($2,A0)
		move.w	D0, ($1b00,A0)	; right
		move.w	D1, ($1b02,A0)
		adda.l	#$4, A0
	dbra	D2, lb_border_row

	movem.l	(A7)+, D0-D2/A0
	rts

; clear ram region
;	D0 size ((size / 4) - 1)
;	A0 address
fn_clear_ram
	move.l	D1, -(A7)
	moveq.l	#$0, D1
lb_clear_ram_loop
		move.l	D1, (A0)+
	dbra	D0, lb_clear_ram_loop
	move.l	(A7)+, D1
	rts


; fill zoom table w/pre-determined values
fn_fill_zoom_table
	movem.l	D0/A0-A1, -(A7)
	move.w	#$1f, D0
	lea.l	lb_zoomtable, A0
	lea.l	addr_zoom_table, A1
_copy_zoomtable
		move.w	(A0)+, (A1)+
	dbra	D0, _copy_zoomtable
	movem.l	(A7)+, D0/A0-A1
	rts


; update fruit position
fn_update_fruit_pos	; rng
	movem.l	D0-D1/A0, -(A7)

	; get a rng'd y position
	lea.l	lb_rng_high, A0
	move.w	pos_fruit_y_rng, D0
	add.w	#$1, pos_fruit_y_rng
	move.w	D0, D1
	ror.w	#$8, D1
	ror.b	#$4, D1
	eor.b	D1, D0
	andi.w	#$ff, D0
	move.b	(A0,D0.w), D0
	move.w	pos_fruit_y, pos_fruit_y_prev
	move.w	D0, pos_fruit_y

	; get a rng'd x position
	lea.l	lb_rng_wide, A0
	move.w	pos_fruit_x_rng, D0
	add.w	#$1, pos_fruit_x_rng
	move.w	D0, D1
	ror.w	#$8, D1
	ror.b	#$3, D1
	eor.b	D1, D0
	andi.w	#$ff, D0
	move.b	(A0,D0.w), D0
	move.w	pos_fruit_x, pos_fruit_x_prev
	move.w	D0, pos_fruit_x

	; clear old fruit position on screen
	lea.l	addr_char_ram, A0
	moveq.l	#$0, D0
	move.w	pos_fruit_y_prev, D0
	lsl.w	#$6, D0
	add.w	pos_fruit_x_prev, D0
	lsl.w	#$2, D0
	move.w	#$0, (A0,D0.w)

	; set to draw new fruit position
	move.w	pos_fruit_y, D0
	lsl.w	#$6, D0
	add.w	pos_fruit_x, D0
	lsl.w	#$2, D0
	move.w	#$20, (A0,D0.w)

	movem.l	(A7)+, D0-D1/A0
	rts


; check collision between two objects
; 	D0 = source 1 start pos y
; 	D1 = source 1 start pos x
; 	D2 = source 2 start pos y
; 	D3 = source 2 start pos x
; 	D4 = (source 1 size y) | ((source 1 size x) << 16)
; 	D5 = (source 2 size y) | ((source 2 size x) << 16)
; 	D6 = temp
fn_check_collision
	move.w	D0, D6
	cmp.w	D6, D2
	blt.s	lb_not_y_less
		add.w	D4, D6	; add source 1 height
		cmp.w	D6, D2
		bge.s	lb_not_y_less
			bra.s	lb_check_x_collision ; already collided 'y', don't bother with second check
lb_not_y_less

	move.w	D0, D6
	add.w	D5, D2	; add source 2 height
	cmp.w	D6, D2
	blt.s	lb_no_collision
		add.w	D4, D6	; add source 1 height
		cmp.w	D6, D2
		bge.s	lb_no_collision

lb_check_x_collision
	swap	D4		; x size (source 1) is high word

	move.w	D1, D6
	cmp.w	D6, D3
	blt.s	lb_not_x_less
		add.w	D4, D6 ; add source 1 width
		cmp.w	D6, D3
		bge.s	lb_not_x_less
			moveq.l	#$1, D6	; collided
			rts
lb_not_x_less

	swap	D5		; x size (source 2) is high word

	move.w	D1, D6
	add.w	D5, D3 	; add source 2 width
	cmp.w	D6, D3
	blt.s	lb_no_collision
		add.w	D4, D6 ; add source 1 width
		cmp.w	D6, D3
		bge.s	lb_no_collision
			moveq.l	#$1, D6	; collided
			rts
lb_no_collision
	moveq.l	#$0, D6
	rts


; print a character to screen
;	A0 = source
;	A1 = dest		- (sy * 256) + (sx * 4);
;	D0 = color
fn_print
	movem.l	D1, -(A7)
	moveq.l	#$0, D1
lb_copy_char
		move.b	(A0)+, D1
		tst.b	D1
		beq.s	lb_end_char_copy
		addq.b	#$8, D1
		move.w	D1, (A1)+
		move.w	D0, (A1)+
	bra.s	lb_copy_char
lb_end_char_copy
	movem.l	(A7)+, D1
	rts

; print a character to screen using double-width font
;	A0 = source
;	A1 = dest		- (sy * 256) + (sx * 4);
;	D0 = color
fn_print_double_wide
	movem.l	D1-D2, -(A7)
	moveq.l	#$0, D1
	move.w	#$28, D2
lb_copy_char_double_wide
		moveq.l	#$0, D1
		move.b	(A0)+, D1
		tst.b	D1
		add.w	D1, D1
		beq.s	lb_end_char_copy_double_wide
		add.w	D2, D1
		move.w	D1, (A1)+
		move.w	D0, (A1)+
		addq.w	#$1, D1
		move.w	D1, (A1)+
		move.w	D0, (A1)+
	bra.s	lb_copy_char_double_wide
lb_end_char_copy_double_wide
	movem.l	(A7)+, D1-D2
	rts

; print a character to screen using double-height font
;	A0 = source
;	A1 = dest		- (sy * 256) + (sx * 4);
;	D0 = color
fn_print_double_height
	movem.l	D1-D2, -(A7)
	moveq.l	#$0, D1
	move.w	#$a9, D2
lb_copy_char_double_height
		moveq.l	#$0, D1
		move.b	(A0)+, D1
		tst.b	D1
		beq.s	lb_end_char_copy_double_height
		add.w	D1, D1
		add.w	D2, D1
		move.w	D1, ($100,A1)
		subq.w	#$1, D1
		move.w	D1, (A1)+
		move.w	D0, ($100,A1)
		move.w	D0, (A1)+
	bra.s	lb_copy_char_double_height
lb_end_char_copy_double_height
	movem.l	(A7)+, D1-D2
	rts

; print a character to screen using double-sized (16x16) font
;	A0 = source
;	A1 = dest		- (sy * 256) + (sx * 4);
;	D0 = color
fn_print_double_size
	movem.l	D1-D2, -(A7)
	moveq.l	#$0, D1
	move.w	#$eb, D2
lb_copy_char_double_size
		moveq.l	#$0, D1
		move.b	(A0)+, D1
		tst.b	D1
		beq.s	lb_end_char_copy_double_height
		lsl.w	#$2, D1
		add.w	D2, D1

		move.w	D1, ($104,A1)
		move.w	D0, ($106,A1)
		subq.w	#$1, D1
		move.w	D1, ($100,A1)
		move.w	D0, ($102,A1)
		subq.w	#$1, D1
		move.w	D1, ($4,A1)
		move.w	D0, ($6,A1)
		subq.w	#$1, D1
		move.w	D1, (A1)
		move.w	D0, ($2,A1)
		adda.l	#$8, A1
	bra.s	lb_copy_char_double_size
lb_end_char_copy_double_size
	movem.l	(A7)+, D1-D2
	rts

; print a character to screen using triple-sized (24x24) font
;	A0 = source
;	A1 = dest		- (sy * 256) + (sx * 4);
;	D0 = color
fn_print_triple_size
	movem.l	D1-D3, -(A7)
	moveq.l	#$0, D1
	move.w	#$150, D2	; right!
	move.w	#$9, D3
lb_copy_char_triple_size
		moveq.l	#$0, D1
		move.b	(A0)+, D1
		tst.b	D1
		beq.s	lb_end_char_copy_triple_size
		mulu.w	D3, D1
		add.w	D2, D1

		move.w	D1, ($208,A1)
		move.w	D0, ($20a,A1)
		subq.w	#$1, D1
		move.w	D1, ($204,A1)
		move.w	D0, ($206,A1)
		subq.w	#$1, D1
		move.w	D1, ($200,A1)
		move.w	D0, ($202,A1)
		subq.w	#$1, D1

		move.w	D1, ($108,A1)
		move.w	D0, ($10a,A1)
		subq.w	#$1, D1
		move.w	D1, ($104,A1)
		move.w	D0, ($106,A1)
		subq.w	#$1, D1
		move.w	D1, ($100,A1)
		move.w	D0, ($102,A1)
		subq.w	#$1, D1

		move.w	D1, ($8,A1)
		move.w	D0, ($0a,A1)
		subq.w	#$1, D1
		move.w	D1, ($4,A1)
		move.w	D0, ($6,A1)
		subq.w	#$1, D1
		move.w	D1, (A1)
		move.w	D0, ($2,A1)
		adda.l	#$c, A1
	bra.s	lb_copy_char_triple_size
lb_end_char_copy_triple_size
	movem.l	(A7)+, D1-D3
	rts

; print a character to screen using quad-sized (32x32) font
;	A0 = source
;	A1 = dest		- (sy * 256) + (sx * 4);
;	D0 = color
fn_print_quad_size
	movem.l	D1-D2, -(A7)
	moveq.l	#$0, D1
	move.w	#$2b7, D2	; right!
lb_copy_char_quad_size
		moveq.l	#$0, D1
		move.b	(A0)+, D1
		tst.b	D1
		beq		lb_end_char_copy_quad_size
		lsl.w	#$4, D1
		add.w	D2, D1

		move.w	D1, ($30c,A1)
		move.w	D0, ($30e,A1)
		subq.w	#$1, D1
		move.w	D1, ($308,A1)
		move.w	D0, ($30a,A1)
		subq.w	#$1, D1
		move.w	D1, ($304,A1)
		move.w	D0, ($306,A1)
		subq.w	#$1, D1
		move.w	D1, ($300,A1)
		move.w	D0, ($302,A1)
		subq.w	#$1, D1

		move.w	D1, ($20c,A1)
		move.w	D0, ($20e,A1)
		subq.w	#$1, D1
		move.w	D1, ($208,A1)
		move.w	D0, ($20a,A1)
		subq.w	#$1, D1
		move.w	D1, ($204,A1)
		move.w	D0, ($206,A1)
		subq.w	#$1, D1
		move.w	D1, ($200,A1)
		move.w	D0, ($202,A1)
		subq.w	#$1, D1

		move.w	D1, ($10c,A1)
		move.w	D0, ($10e,A1)
		subq.w	#$1, D1
		move.w	D1, ($108,A1)
		move.w	D0, ($10a,A1)
		subq.w	#$1, D1
		move.w	D1, ($104,A1)
		move.w	D0, ($106,A1)
		subq.w	#$1, D1
		move.w	D1, ($100,A1)
		move.w	D0, ($102,A1)
		subq.w	#$1, D1

		move.w	D1, ($0c,A1)
		move.w	D0, ($0e,A1)
		subq.w	#$1, D1
		move.w	D1, ($8,A1)
		move.w	D0, ($0a,A1)
		subq.w	#$1, D1
		move.w	D1, ($4,A1)
		move.w	D0, ($6,A1)
		subq.w	#$1, D1
		move.w	D1, (A1)
		move.w	D0, ($2,A1)
		adda.l	#$10, A1
	bra		lb_copy_char_quad_size
lb_end_char_copy_quad_size
	movem.l	(A7)+, D1-D2
	rts


; print a hex/decimal character to screen
; 	A0 = dest		- (sy * 256) + (sx * 4);
; 	D0 = source
; 	D1 = color
; 	D2 = digits
fn_print_hex
	movem.l	D0-D3, -(A7)
	moveq.l	#$0, D3
	subq.l	#$1, D2	; for dbra
	lsl.w	#$2, D2	; for now
	ror.l	D2, D0
	lsr.w	#$2, D2 ; back!
lb_copy_hex
		move.b	D0, D3
		andi.b	#$f, D3
		rol.l	#$4, D0
		cmpi.b	#$9, D3
		ble.s	lb_hex_not_10
			addq.l	#$7, D3
lb_hex_not_10
		addi.w	#$38, D3
		move.w	D3, (A0)+
		move.w	D1, (A0)+

	dbra	D2, lb_copy_hex
	movem.l	(A7)+, D0-D3
	rts

lb_testtext
	dc.b	'TEST', 0, 0

lb_zoomtable
	dc.w $5555, $5555, $5515, $5555, $5515, $5515, $1515, $5515, $1515, $1515, $1511, $1515, $1511, $1511, $1111, $1511 
	dc.w $1111, $1111, $1101, $1111, $1101, $1101, $0101, $1101, $0101, $0101, $0001, $0101, $0001, $0001, $0000, $0001

lb_rng_wide
	dc.b	$23, $16, $12, $19, $03, $23, $18, $21, $0D, $1B, $04, $22, $15, $02, $02, $14
	dc.b	$15, $16, $21, $0D, $11, $19, $23, $07, $22, $01, $0A, $07, $04, $22, $0A, $1F
	dc.b	$23, $03, $16, $1C, $13, $23, $04, $01, $07, $0E, $17, $11, $1D, $17, $15, $10
	dc.b	$16, $1E, $01, $18, $02, $11, $1F, $07, $24, $01, $0F, $1A, $19, $0E, $11, $19
	dc.b	$02, $02, $14, $06, $20, $1C, $23, $23, $0F, $02, $01, $1F, $0F, $0E, $03, $1C
	dc.b	$07, $08, $02, $09, $23, $0F, $1D, $1E, $1E, $0D, $1C, $13, $0E, $19, $05, $06
	dc.b	$0A, $02, $1A, $07, $19, $1C, $24, $17, $09, $11, $17, $08, $23, $09, $10, $04
	dc.b	$09, $12, $18, $21, $23, $01, $07, $0A, $0D, $19, $1C, $23, $19, $1A, $15, $23
	dc.b	$1F, $16, $16, $14, $13, $03, $19, $01, $1A, $1E, $0F, $15, $10, $12, $1C, $1A
	dc.b	$1F, $11, $16, $1D, $20, $14, $07, $21, $04, $0A, $0E, $09, $15, $17, $0E, $0F
	dc.b	$07, $08, $1B, $0E, $1F, $16, $0E, $0A, $11, $1F, $1F, $1E, $09, $19, $17, $12
	dc.b	$0A, $21, $1B, $12, $11, $01, $1D, $06, $0D, $1E, $04, $10, $10, $1D, $0F, $0F
	dc.b	$23, $12, $1B, $08, $07, $01, $0C, $1B, $20, $1D, $02, $1C, $0B, $15, $1E, $22
	dc.b	$24, $18, $23, $23, $0C, $09, $22, $19, $03, $17, $04, $16, $06, $22, $22, $16
	dc.b	$24, $22, $0D, $23, $22, $1A, $11, $1D, $01, $0F, $19, $05, $21, $08, $12, $08
	dc.b	$1B, $13, $0A, $16, $06, $1C, $08, $09, $23, $0F, $02, $15, $01, $23, $14, $07

lb_rng_high
	dc.b	$13, $16, $06, $16, $08, $08, $0B, $0E, $0D, $16, $06, $0D, $14, $09, $16, $03
	dc.b	$16, $0E, $1A, $14, $09, $0C, $04, $12, $0F, $17, $08, $06, $15, $02, $1A, $15
	dc.b	$0B, $0A, $10, $18, $17, $11, $10, $04, $0F, $13, $01, $18, $13, $06, $08, $05
	dc.b	$05, $16, $03, $17, $16, $18, $0E, $04, $04, $05, $01, $07, $0A, $05, $04, $0F
	dc.b	$16, $12, $08, $06, $0B, $17, $16, $19, $15, $07, $17, $14, $07, $02, $0A, $14
	dc.b	$12, $0E, $10, $01, $13, $19, $08, $13, $04, $01, $07, $06, $15, $19, $08, $10
	dc.b	$1A, $11, $03, $0F, $18, $0C, $15, $03, $13, $03, $18, $11, $14, $15, $10, $02
	dc.b	$08, $09, $17, $02, $15, $01, $14, $03, $18, $0F, $14, $12, $19, $1A, $05, $0E
	dc.b	$06, $03, $06, $0B, $08, $0F, $11, $0D, $15, $08, $02, $0D, $07, $08, $0B, $11
	dc.b	$0B, $16, $12, $14, $18, $0E, $0E, $13, $0B, $06, $0B, $0A, $0C, $06, $10, $1A 
	dc.b	$0A, $19, $01, $05, $19, $02, $03, $0D, $08, $0A, $1A, $0E, $0B, $19, $0A, $16
	dc.b	$13, $0F, $14, $0F, $03, $08, $16, $0C, $15, $01, $10, $10, $10, $13, $08, $14
	dc.b	$10, $17, $18, $01, $01, $0E, $0C, $01, $0E, $05, $17, $0B, $0F, $01, $19, $07
	dc.b	$08, $01, $04, $17, $0E, $03, $0A, $16, $18, $0B, $15, $14, $06, $14, $01, $12
	dc.b	$0E, $11, $15, $1A, $03, $05, $09, $11, $0E, $06, $02, $0f, $10, $07, $17, $0F
	dc.b	$15, $13, $0E, $16, $0D, $08, $0C, $18, $0F, $06, $15, $14, $18, $15, $13, $11

    END    START        ; last line of source







*~Font name~Courier New~
*~Font size~14~
*~Tab type~1~
*~Tab size~4~
